/*
  File: Diners.java
*/

package diners;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;

public class Diners extends Applet 
{

	PhilTable table_;

	Scrollbar slider_;

	Philosopher phils_[] = new Philosopher[PhilTable.NUMPHILS];

	RunnablePhilosopher rphils_[] = 
		new RunnablePhilosopher[PhilTable.NUMPHILS];

	Fork forks_[] = new Fork[PhilTable.NUMPHILS];
 
	public void init() 
	{
		setLayout( new BorderLayout() );
		table_ = new PhilTable( this );
		table_.disableUpdate();
		add( table_, BorderLayout.CENTER );

		Panel p = new Panel( new BorderLayout() );

		slider_ = new Scrollbar( Scrollbar.HORIZONTAL, 50, 5, 0, 100 );
		p.add( slider_, BorderLayout.CENTER );

		Button r = new Button( "Restart" );
		r.addActionListener( 
			  new ActionListener() 
			  {
				  public void actionPerformed( ActionEvent e ) 
					  {
						  restart(); 
					  }
			  } ); 
		p.add( r, BorderLayout.EAST );

		Button f = new Button( "Stop" );
		f.addActionListener(
			  new ActionListener() 
			  {
				  public void actionPerformed( ActionEvent e ) 
					  {
						  freeze(); 
					  }
			  } ); 
		p.add( f, BorderLayout.WEST );
		add( p, BorderLayout.SOUTH );
	}

	private void setupPhilosophers()
	{
		for ( int i = 0; i < PhilTable.NUMPHILS; i++ )
			{
				Fork l = forks_[(i-1+PhilTable.NUMPHILS)%PhilTable.NUMPHILS];
				Fork r = forks_[i];

				phils_[i] = new Philosopher( table_, i, l, r );
				rphils_[i] = new RunnablePhilosopher( phils_[i], this );
			}
	}

	private void setupForks()
	{
		for ( int i = 0; i < PhilTable.NUMPHILS; i++ )
			forks_[i] = new Fork( table_, i );
	}
	
	public int getThinkTime() 
	{
		return (slider_.getValue() * (int)(100 * Math.random()));
	}

	public int getEatTime() 
	{
		return (slider_.getValue() * (int)(50 * Math.random()));
	}

	public PhilTable getView()
	{
		return table_;
	}

	public void start() 
	{
		setupForks();
		setupPhilosophers();
		table_.start();
		table_.enableUpdate();
		for ( int i = 0; i < PhilTable.NUMPHILS; i++ )
			rphils_[i].start();
	}

	public synchronized void stop() 
	{
		// terminate threads
		for ( int i = 0; i < PhilTable.NUMPHILS; i++ )
			{
				if ( rphils_[i] != null )
					if ( rphils_[i].isAlive() )
						{
							rphils_[i].terminate();
							rphils_[i] = null;
						}
			}
		notifyAll();
	}

	void restart() 
	{
		stop();
		slider_.setValue( 50 );
		table_.disableUpdate();
		//		for ( int i = 0; i < PhilTable.NUMPHILS; i++ )
		//			rphils_[i] = new RunnablePhilosopher( phils_[i], this );
		start();
	}

	void freeze() 
	{
		stop();
		table_.setFrozen();
	}

}
