package bridgeSol;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*; 

public class SingleLaneBridge extends Applet 
{

	BridgeCanvas display_;

	Choice bridgePolicyChooser_;  
	final static String FAIR = "fair"; 
	final static String SAFE = "safe"; 
	final static String UNSAFE = "unsafe"; 
	int maxCar = 2;

	Vector cars_;  

	public void init() 
	{
		setLayout( new BorderLayout() );
		display_ = new BridgeCanvas( this );
		add( display_, BorderLayout.CENTER );
		Button restart = new Button( "Restart" );
		restart.addActionListener( new RestartListener( this )); 

		Button freeze = new Button( "Freeze" );
		freeze.addActionListener( new FreezeListener( this )); 
		Button onecar = new Button( "One Car" );
		onecar.addActionListener( new CarsListener( this, 1 )); 
		Button twocar = new Button( "Two Cars" );
		twocar.addActionListener( new CarsListener( this, 2 )); 
		Button threecar = new Button( "Three Cars" );
		threecar.addActionListener( new CarsListener( this, 3 )); 

		bridgePolicyChooser_ = new Choice(); 
		bridgePolicyChooser_.add( UNSAFE ); 
		bridgePolicyChooser_.add( SAFE ); 
		bridgePolicyChooser_.add( FAIR ); 

		Panel ptop = new Panel();
		ptop.setLayout( new FlowLayout());
		ptop.add( freeze ); 
		ptop.add( restart );
		ptop.add( bridgePolicyChooser_ );
		add( ptop, BorderLayout.NORTH );

		Panel pbot = new Panel();
		pbot.add( onecar );
		pbot.add( twocar );
		pbot.add( threecar );
		add( pbot, BorderLayout.SOUTH );

		cars_ = new Vector(); 
	}


	public void start() 
	{
		cars_.removeAllElements(); 
		String chosenPolicy = bridgePolicyChooser_.getSelectedItem(); 
		display_.init(chosenPolicy );
		IBridge b;

		if ( chosenPolicy.equals(FAIR )) 
			b = new FairBridge();
		else if ( chosenPolicy.equals(SAFE )) 
			b = new SafeBridge();
		else
			b = new DefaultBridge();
		for (int i = 0; i<maxCar; i++) 
		{
			Car c = new RedCar(b,display_ ); 
			display_.registerCar(c ); 
			cars_.addElement( c ); 
			c = new BlueCar( b,display_ ); 
			display_.registerCar( c ); 
			cars_.addElement( c ); 
		}

		Enumeration en = cars_.elements(); 
		while ( en.hasMoreElements()) 
		{
			Car c = (Car)en.nextElement(); 
			c.start(); 
		}
	}

	public void stop() 
	{
		Enumeration en = cars_.elements(); 
		while ( en.hasMoreElements()) 
		{
			Car c = (Car)en.nextElement(); 
			c.stop(); 
		}
	}


	public void restart() 
	{
		stop(); 
		start(); 
	}

	public void freeze() 
	{
		stop(); 
	}

	public void setMaxCar(int n) 
	{
		stop();
		maxCar = n;
		start();
	}


	class FreezeListener implements ActionListener
	{

		final SingleLaneBridge owner_;

		public FreezeListener( SingleLaneBridge owner )
		{
			owner_ = owner;
		}


		public void actionPerformed( ActionEvent evt )
		{
			owner_.stop(); 
		}

	}
	class RestartListener implements ActionListener
	{

		final SingleLaneBridge owner_;

		public RestartListener( SingleLaneBridge owner )
		{
			owner_ = owner;
		}


		public void actionPerformed( ActionEvent evt )
		{
			owner_.stop();
			owner_.start();
		}

	}

	class CarsListener implements ActionListener
	{

		final SingleLaneBridge owner_;
		int number_; 

		public CarsListener( SingleLaneBridge owner, int number )
		{
			owner_ = owner;
			number_ = number; 
		}


		public void actionPerformed( ActionEvent evt )
		{
			owner_.setMaxCar(number_); 
		}

	}

}

