/*
  File: BufferCanvas.java
*/

package bufferSol;

import java.awt.*;

class BufferPanel extends Panel
{

	Font font_ = new Font("Times",Font.ITALIC+Font.BOLD,24);

	Label title_;

	BufferCanvas bufCanvas_;

	public BufferPanel( String title, int slots )
	{
		this( new BorderLayout(), title, Color.orange, slots );
	}

	public BufferPanel( String title, Color bgc, int slots )
	{
		this( new BorderLayout(), title, bgc, slots );
	}

	public BufferPanel( LayoutManager layout, String title, int slots )
	{
		this( layout, title, Color.orange, slots );
	}

	public BufferPanel( LayoutManager layout, String title, 
						Color bgc, int slots )
	{
		super( layout );
		setBackground( bgc );

		title_ = new Label( title, Label.CENTER );
		title_.setFont( font_ );
		add( title_, BorderLayout.NORTH );

		bufCanvas_ = new BufferCanvas( slots, bgc );
		add( bufCanvas_, BorderLayout.CENTER );
	}

	public void setValue( char[] buf, int in, int out )
	{
		bufCanvas_.setValue( buf, in, out );
	}

}

class BufferCanvas extends Canvas 
{

	int slots_;

	int in_ = 0;

	int out_ = 0;

	char[] buf_;

	Font font_ = new Font( "TimesRoman", Font.BOLD, 36 );

	BufferCanvas( int slots, Color bgc ) 
	{
		super();
		slots_ = slots;
		buf_ = new char[slots];
		for ( int i = 0; i < slots; i++ )
			buf_[i] = ' ';
		setSize( 20 + 50 * slots_, 144 );
		setBackground( bgc );
	}

	public void setValue( char[] buf, int in, int out )
	{
		buf_ = buf;
		in_ = in;
		out_ = out;
		repaint();
	}

	public void paint( Graphics g ) 
	{
		update( g );
	}

	Image offscreen_;
	Dimension offscreensize_;
	Graphics offgraphics_;

	public synchronized void update( Graphics g )
	{
		// prepare offgraphics
		Dimension d = getSize();
		if ( (offscreen_ == null) || 
			 (d.width != offscreensize_.width) || 
			 (d.height != offscreensize_.height) ) 
			{
				offscreen_ = createImage(d.width, d.height);
				offscreensize_ = d;
				offgraphics_ = offscreen_.getGraphics();
				offgraphics_.setFont( getFont() );
			}

		offgraphics_.setColor( getBackground() );
		offgraphics_.fillRect( 0, 0, d.width, d.height );

		// Buffer Boxes
		int y = d.height/2 - 25;
		offgraphics_.setColor( Color.white );
		offgraphics_.fillRect( 10, y, 50 * slots_, 50 );
		offgraphics_.setColor( Color.black );
		offgraphics_.setFont( font_ );
		for( int i = 0; i < slots_; i++ ) 
			{
				offgraphics_.drawRect( 10 + 50 * i, y, 50, 50 );
				offgraphics_.drawChars( buf_, i, 1, 25 + 50 * i, y + 35 );
			}
		
		//Input and output Pointers
		offgraphics_.setColor( Color.blue );
		offgraphics_.fillOval( 28 + 50 * in_, y - 20, 15, 15 );
		offgraphics_.setColor( Color.yellow );
		offgraphics_.fillOval( 28 + 50 * out_, y + 55, 15, 15 );
		g.drawImage( offscreen_, 0, 0, null );
	}

}

